#ifndef LITTLE_HEADER
#define LITTLE_HEADER 1
#include <stdlib.h>
#include <stdint.h>

typedef struct little_struct *little_t;

/**
 * Initialize the LittleUSB system
 */
void little_init();

/**
 * Set the method to log diagnostics to, eg little_setlog(&printf)
 * which is the default
 */
void little_setlog(int (*printf)(const char *format, ...));


/**
 * Reset all LittleUSB devices (not really needed, just unplug them)
 */
void little_resetall();

/**
 * Shutdown the LittleUSB system
 */
void little_shutdown();

/**
 * List the LittleUSB devices available. The addresslist is
 * populated with up to *max* matching addresses, and
 * the number of addresses is returned. Each address is 16 bits,
 * with the high 8-bits the USB bus number and the low 8-bits the USB device.
 * - addresslist - an array of *max* addresses which will be populated
 * - max - the length of the array *addresslist*
 * returns the number of matching items found from 0..max-1
 */
size_t little_list(uint16_t *addresslist, size_t max);

/**
 * Open a LittleUSB device
 * - address - an address, normally as returned from littie_list
 * returns the device or NULL if it failed to open
 */
little_t little_open(uint16_t address);

/**
 * Close a previously opened LittleUSB device
 */
void little_close(little_t s);

/**
 * Free any memory allocated for a previously closed LittleUSB device
 */
void little_free(little_t s);

/**
 * Return the state of the LittleUSB device. The state is a bitmask with
 * the following values:
 * bit 0 - input (as set by previous component - default value is 0)
 * bit 1 - output (as passed to next component and set by litle_set)
 * bit 2 - LED (as set by little_set)
 * returns < 0 on error.
 */
int little_get(little_t s);

/**
 * Set the state of the LittleUSB device. The state is a bitmask with
 * the following values:
 * bit 1 - output (as passed to next component - default value is 0)
 * bit 2 - LED (1 for on, 0 for off)
 * Setting other bits will have no effect.
 * returns < 0 on error
 */
int little_set(little_t s, int value);

#endif
